/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __TRACKER_INDEXER_PROXY_H__
#define __TRACKER_INDEXER_PROXY_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.Tracker3.Miner */

#define TRACKER_INDEXER_TYPE_MINER (tracker_indexer_miner_get_type ())
#define TRACKER_INDEXER_MINER(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_INDEXER_TYPE_MINER, TrackerIndexerMiner))
#define TRACKER_INDEXER_IS_MINER(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_INDEXER_TYPE_MINER))
#define TRACKER_INDEXER_MINER_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TRACKER_INDEXER_TYPE_MINER, TrackerIndexerMinerIface))

struct _TrackerIndexerMiner;
typedef struct _TrackerIndexerMiner TrackerIndexerMiner;
typedef struct _TrackerIndexerMinerIface TrackerIndexerMinerIface;

struct _TrackerIndexerMinerIface
{
  GTypeInterface parent_iface;


  gboolean (*handle_get_pause_details) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_progress) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_remaining_time) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_get_status) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);

  gboolean (*handle_pause) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_application,
    const gchar *arg_reason);

  gboolean (*handle_pause_for_process) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_application,
    const gchar *arg_reason);

  gboolean (*handle_resume) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    gint arg_cookie);

  gboolean (*handle_start) (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);

  void (*paused) (
    TrackerIndexerMiner *object);

  void (*progress) (
    TrackerIndexerMiner *object,
    const gchar *arg_status,
    gdouble arg_progress,
    gint arg_remaining_time);

  void (*resumed) (
    TrackerIndexerMiner *object);

  void (*started) (
    TrackerIndexerMiner *object);

  void (*stopped) (
    TrackerIndexerMiner *object,
    gboolean arg_interrupted);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackerIndexerMiner, g_object_unref)
#endif

GType tracker_indexer_miner_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *tracker_indexer_miner_interface_info (void);
guint tracker_indexer_miner_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void tracker_indexer_miner_complete_start (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);

void tracker_indexer_miner_complete_get_status (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    const gchar *status);

void tracker_indexer_miner_complete_get_progress (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    gdouble progress);

void tracker_indexer_miner_complete_get_remaining_time (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    gint remaining_time);

void tracker_indexer_miner_complete_get_pause_details (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    const gchar *const *pause_applications,
    const gchar *const *pause_reasons);

void tracker_indexer_miner_complete_pause (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    gint cookie);

void tracker_indexer_miner_complete_pause_for_process (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation,
    gint cookie);

void tracker_indexer_miner_complete_resume (
    TrackerIndexerMiner *object,
    GDBusMethodInvocation *invocation);



/* D-Bus signal emissions functions: */
void tracker_indexer_miner_emit_started (
    TrackerIndexerMiner *object);

void tracker_indexer_miner_emit_stopped (
    TrackerIndexerMiner *object,
    gboolean arg_interrupted);

void tracker_indexer_miner_emit_paused (
    TrackerIndexerMiner *object);

void tracker_indexer_miner_emit_resumed (
    TrackerIndexerMiner *object);

void tracker_indexer_miner_emit_progress (
    TrackerIndexerMiner *object,
    const gchar *arg_status,
    gdouble arg_progress,
    gint arg_remaining_time);



/* D-Bus method calls: */
void tracker_indexer_miner_call_start (
    TrackerIndexerMiner *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_start_finish (
    TrackerIndexerMiner *proxy,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_start_sync (
    TrackerIndexerMiner *proxy,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_get_status (
    TrackerIndexerMiner *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_get_status_finish (
    TrackerIndexerMiner *proxy,
    gchar ** out_status,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_get_status_sync (
    TrackerIndexerMiner *proxy,
    gchar ** out_status,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_get_progress (
    TrackerIndexerMiner *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_get_progress_finish (
    TrackerIndexerMiner *proxy,
    gdouble* out_progress,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_get_progress_sync (
    TrackerIndexerMiner *proxy,
    gdouble* out_progress,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_get_remaining_time (
    TrackerIndexerMiner *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_get_remaining_time_finish (
    TrackerIndexerMiner *proxy,
    gint* out_remaining_time,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_get_remaining_time_sync (
    TrackerIndexerMiner *proxy,
    gint* out_remaining_time,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_get_pause_details (
    TrackerIndexerMiner *proxy,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_get_pause_details_finish (
    TrackerIndexerMiner *proxy,
    gchar *** out_pause_applications,
    gchar *** out_pause_reasons,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_get_pause_details_sync (
    TrackerIndexerMiner *proxy,
    gchar *** out_pause_applications,
    gchar *** out_pause_reasons,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_pause (
    TrackerIndexerMiner *proxy,
    const gchar *arg_application,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_pause_finish (
    TrackerIndexerMiner *proxy,
    gint* out_cookie,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_pause_sync (
    TrackerIndexerMiner *proxy,
    const gchar *arg_application,
    const gchar *arg_reason,
    gint* out_cookie,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_pause_for_process (
    TrackerIndexerMiner *proxy,
    const gchar *arg_application,
    const gchar *arg_reason,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_pause_for_process_finish (
    TrackerIndexerMiner *proxy,
    gint* out_cookie,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_pause_for_process_sync (
    TrackerIndexerMiner *proxy,
    const gchar *arg_application,
    const gchar *arg_reason,
    gint* out_cookie,
    GCancellable *cancellable,
    GError **error);

void tracker_indexer_miner_call_resume (
    TrackerIndexerMiner *proxy,
    gint arg_cookie,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_indexer_miner_call_resume_finish (
    TrackerIndexerMiner *proxy,
    GAsyncResult *res,
    GError **error);

gboolean tracker_indexer_miner_call_resume_sync (
    TrackerIndexerMiner *proxy,
    gint arg_cookie,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define TRACKER_INDEXER_TYPE_MINER_PROXY (tracker_indexer_miner_proxy_get_type ())
#define TRACKER_INDEXER_MINER_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_INDEXER_TYPE_MINER_PROXY, TrackerIndexerMinerProxy))
#define TRACKER_INDEXER_MINER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TRACKER_INDEXER_TYPE_MINER_PROXY, TrackerIndexerMinerProxyClass))
#define TRACKER_INDEXER_MINER_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TRACKER_INDEXER_TYPE_MINER_PROXY, TrackerIndexerMinerProxyClass))
#define TRACKER_INDEXER_IS_MINER_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_INDEXER_TYPE_MINER_PROXY))
#define TRACKER_INDEXER_IS_MINER_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TRACKER_INDEXER_TYPE_MINER_PROXY))

typedef struct _TrackerIndexerMinerProxy TrackerIndexerMinerProxy;
typedef struct _TrackerIndexerMinerProxyClass TrackerIndexerMinerProxyClass;
typedef struct _TrackerIndexerMinerProxyPrivate TrackerIndexerMinerProxyPrivate;

struct _TrackerIndexerMinerProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  TrackerIndexerMinerProxyPrivate *priv;
};

struct _TrackerIndexerMinerProxyClass
{
  GDBusProxyClass parent_class;
};

GType tracker_indexer_miner_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackerIndexerMinerProxy, g_object_unref)
#endif

void tracker_indexer_miner_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
TrackerIndexerMiner *tracker_indexer_miner_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
TrackerIndexerMiner *tracker_indexer_miner_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void tracker_indexer_miner_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
TrackerIndexerMiner *tracker_indexer_miner_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
TrackerIndexerMiner *tracker_indexer_miner_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TRACKER_INDEXER_TYPE_MINER_SKELETON (tracker_indexer_miner_skeleton_get_type ())
#define TRACKER_INDEXER_MINER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_INDEXER_TYPE_MINER_SKELETON, TrackerIndexerMinerSkeleton))
#define TRACKER_INDEXER_MINER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TRACKER_INDEXER_TYPE_MINER_SKELETON, TrackerIndexerMinerSkeletonClass))
#define TRACKER_INDEXER_MINER_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TRACKER_INDEXER_TYPE_MINER_SKELETON, TrackerIndexerMinerSkeletonClass))
#define TRACKER_INDEXER_IS_MINER_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_INDEXER_TYPE_MINER_SKELETON))
#define TRACKER_INDEXER_IS_MINER_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TRACKER_INDEXER_TYPE_MINER_SKELETON))

typedef struct _TrackerIndexerMinerSkeleton TrackerIndexerMinerSkeleton;
typedef struct _TrackerIndexerMinerSkeletonClass TrackerIndexerMinerSkeletonClass;
typedef struct _TrackerIndexerMinerSkeletonPrivate TrackerIndexerMinerSkeletonPrivate;

struct _TrackerIndexerMinerSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  TrackerIndexerMinerSkeletonPrivate *priv;
};

struct _TrackerIndexerMinerSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType tracker_indexer_miner_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackerIndexerMinerSkeleton, g_object_unref)
#endif

TrackerIndexerMiner *tracker_indexer_miner_skeleton_new (void);


G_END_DECLS

#endif /* __TRACKER_INDEXER_PROXY_H__ */
