
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "tracker-miner-enum-types.h"
#include "tracker-miner-enums.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "tracker-miner-enums.h" */

GType
tracker_directory_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (TRACKER_DIRECTORY_FLAG_NONE), "TRACKER_DIRECTORY_FLAG_NONE", "none" },
        { C_FLAGS (TRACKER_DIRECTORY_FLAG_RECURSE), "TRACKER_DIRECTORY_FLAG_RECURSE", "recurse" },
        { C_FLAGS (TRACKER_DIRECTORY_FLAG_PRESERVE), "TRACKER_DIRECTORY_FLAG_PRESERVE", "preserve" },
        { C_FLAGS (TRACKER_DIRECTORY_FLAG_IS_VOLUME), "TRACKER_DIRECTORY_FLAG_IS_VOLUME", "is-volume" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("TrackerDirectoryFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
tracker_filter_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (TRACKER_FILTER_FILE), "TRACKER_FILTER_FILE", "file" },
        { C_ENUM (TRACKER_FILTER_DIRECTORY), "TRACKER_FILTER_DIRECTORY", "directory" },
        { C_ENUM (TRACKER_FILTER_PARENT_DIRECTORY), "TRACKER_FILTER_PARENT_DIRECTORY", "parent-directory" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("TrackerFilterType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

