
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include <config-miners.h>

#include "tracker-miners-enum-types.h"
/* enumerations from "tracker-enums.h" */
#include "/build/localsearch/src/build/../localsearch/src/common/tracker-enums.h"
GType
tracker_index_location_flags_get_type (void)
{
  static gsize g_define_type_id = 0;

  if (g_once_init_enter (&g_define_type_id)) {
    static const GEnumValue values[] = {
      { TRACKER_INDEX_LOCATION_FLAGS_NONE, "TRACKER_INDEX_LOCATION_FLAGS_NONE", "none" },
      { 0, NULL, NULL }
    };
    GType type_id =
       g_enum_register_static (g_intern_static_string ("TrackerIndexLocationFlags"), values);

    g_once_init_leave (&g_define_type_id, type_id);
  }

  return g_define_type_id;
}


/* Generated data ends here */

