/*
 * This file is generated by gdbus-codegen, do not modify it.
 *
 * The license of this code is the same as for the D-Bus interface description
 * it was derived from. Note that it links to GLib, so must comply with the
 * LGPL linking clauses.
 */

#ifndef __TRACKER_CONTROL_PROXY_H__
#define __TRACKER_CONTROL_PROXY_H__

#include <gio/gio.h>

G_BEGIN_DECLS


/* ------------------------------------------------------------------------ */
/* Declarations for org.freedesktop.Tracker3.Miner.Files.Index */

#define TRACKER_CONTROL_TYPE_INDEX (tracker_control_index_get_type ())
#define TRACKER_CONTROL_INDEX(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_CONTROL_TYPE_INDEX, TrackerControlIndex))
#define TRACKER_CONTROL_IS_INDEX(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_CONTROL_TYPE_INDEX))
#define TRACKER_CONTROL_INDEX_GET_IFACE(o) (G_TYPE_INSTANCE_GET_INTERFACE ((o), TRACKER_CONTROL_TYPE_INDEX, TrackerControlIndexIface))

struct _TrackerControlIndex;
typedef struct _TrackerControlIndex TrackerControlIndex;
typedef struct _TrackerControlIndexIface TrackerControlIndexIface;

struct _TrackerControlIndexIface
{
  GTypeInterface parent_iface;

  gboolean (*handle_index_location) (
    TrackerControlIndex *object,
    GDBusMethodInvocation *invocation,
    const gchar *arg_file_uri,
    const gchar *const *arg_graphs,
    const gchar *const *arg_flags);

};

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackerControlIndex, g_object_unref)
#endif

GType tracker_control_index_get_type (void) G_GNUC_CONST;

GDBusInterfaceInfo *tracker_control_index_interface_info (void);
guint tracker_control_index_override_properties (GObjectClass *klass, guint property_id_begin);


/* D-Bus method call completion functions: */
void tracker_control_index_complete_index_location (
    TrackerControlIndex *object,
    GDBusMethodInvocation *invocation);



/* D-Bus method calls: */
void tracker_control_index_call_index_location (
    TrackerControlIndex *proxy,
    const gchar *arg_file_uri,
    const gchar *const *arg_graphs,
    const gchar *const *arg_flags,
    GCancellable *cancellable,
    GAsyncReadyCallback callback,
    gpointer user_data);

gboolean tracker_control_index_call_index_location_finish (
    TrackerControlIndex *proxy,
    GAsyncResult *res,
    GError **error);

gboolean tracker_control_index_call_index_location_sync (
    TrackerControlIndex *proxy,
    const gchar *arg_file_uri,
    const gchar *const *arg_graphs,
    const gchar *const *arg_flags,
    GCancellable *cancellable,
    GError **error);



/* ---- */

#define TRACKER_CONTROL_TYPE_INDEX_PROXY (tracker_control_index_proxy_get_type ())
#define TRACKER_CONTROL_INDEX_PROXY(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_CONTROL_TYPE_INDEX_PROXY, TrackerControlIndexProxy))
#define TRACKER_CONTROL_INDEX_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TRACKER_CONTROL_TYPE_INDEX_PROXY, TrackerControlIndexProxyClass))
#define TRACKER_CONTROL_INDEX_PROXY_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TRACKER_CONTROL_TYPE_INDEX_PROXY, TrackerControlIndexProxyClass))
#define TRACKER_CONTROL_IS_INDEX_PROXY(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_CONTROL_TYPE_INDEX_PROXY))
#define TRACKER_CONTROL_IS_INDEX_PROXY_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TRACKER_CONTROL_TYPE_INDEX_PROXY))

typedef struct _TrackerControlIndexProxy TrackerControlIndexProxy;
typedef struct _TrackerControlIndexProxyClass TrackerControlIndexProxyClass;
typedef struct _TrackerControlIndexProxyPrivate TrackerControlIndexProxyPrivate;

struct _TrackerControlIndexProxy
{
  /*< private >*/
  GDBusProxy parent_instance;
  TrackerControlIndexProxyPrivate *priv;
};

struct _TrackerControlIndexProxyClass
{
  GDBusProxyClass parent_class;
};

GType tracker_control_index_proxy_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackerControlIndexProxy, g_object_unref)
#endif

void tracker_control_index_proxy_new (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
TrackerControlIndex *tracker_control_index_proxy_new_finish (
    GAsyncResult        *res,
    GError             **error);
TrackerControlIndex *tracker_control_index_proxy_new_sync (
    GDBusConnection     *connection,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);

void tracker_control_index_proxy_new_for_bus (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GAsyncReadyCallback  callback,
    gpointer             user_data);
TrackerControlIndex *tracker_control_index_proxy_new_for_bus_finish (
    GAsyncResult        *res,
    GError             **error);
TrackerControlIndex *tracker_control_index_proxy_new_for_bus_sync (
    GBusType             bus_type,
    GDBusProxyFlags      flags,
    const gchar         *name,
    const gchar         *object_path,
    GCancellable        *cancellable,
    GError             **error);


/* ---- */

#define TRACKER_CONTROL_TYPE_INDEX_SKELETON (tracker_control_index_skeleton_get_type ())
#define TRACKER_CONTROL_INDEX_SKELETON(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), TRACKER_CONTROL_TYPE_INDEX_SKELETON, TrackerControlIndexSkeleton))
#define TRACKER_CONTROL_INDEX_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_CAST ((k), TRACKER_CONTROL_TYPE_INDEX_SKELETON, TrackerControlIndexSkeletonClass))
#define TRACKER_CONTROL_INDEX_SKELETON_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), TRACKER_CONTROL_TYPE_INDEX_SKELETON, TrackerControlIndexSkeletonClass))
#define TRACKER_CONTROL_IS_INDEX_SKELETON(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), TRACKER_CONTROL_TYPE_INDEX_SKELETON))
#define TRACKER_CONTROL_IS_INDEX_SKELETON_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), TRACKER_CONTROL_TYPE_INDEX_SKELETON))

typedef struct _TrackerControlIndexSkeleton TrackerControlIndexSkeleton;
typedef struct _TrackerControlIndexSkeletonClass TrackerControlIndexSkeletonClass;
typedef struct _TrackerControlIndexSkeletonPrivate TrackerControlIndexSkeletonPrivate;

struct _TrackerControlIndexSkeleton
{
  /*< private >*/
  GDBusInterfaceSkeleton parent_instance;
  TrackerControlIndexSkeletonPrivate *priv;
};

struct _TrackerControlIndexSkeletonClass
{
  GDBusInterfaceSkeletonClass parent_class;
};

GType tracker_control_index_skeleton_get_type (void) G_GNUC_CONST;

#if GLIB_CHECK_VERSION(2, 44, 0)
G_DEFINE_AUTOPTR_CLEANUP_FUNC (TrackerControlIndexSkeleton, g_object_unref)
#endif

TrackerControlIndex *tracker_control_index_skeleton_new (void);


G_END_DECLS

#endif /* __TRACKER_CONTROL_PROXY_H__ */
